@@ -1,19 +1,22 @@
+from authlib.common.errors import ContinueIteration
+
 from .authenticate_client import ClientAuthentication
-from .errors import (
-    OAuth2Error,
-    InvalidScopeError,
-    UnsupportedResponseTypeError,
-    UnsupportedGrantTypeError,
-)
+from .errors import InvalidScopeError
+from .errors import OAuth2Error
+from .errors import UnsupportedGrantTypeError
+from .errors import UnsupportedResponseTypeError
+from .requests import JsonRequest
+from .requests import OAuth2Request
 from .util import scope_to_list
 
 
-class AuthorizationServer(object):
+class AuthorizationServer:
     """Authorization server that handles Authorization Endpoint and Token
     Endpoint.
 
     :param scopes_supported: A list of supported scopes by this authorization server.
     """
+
     def __init__(self, scopes_supported=None):
         self.scopes_supported = scopes_supported
         self._token_generators = {}
@@ -33,8 +36,15 @@ class AuthorizationServer(object):
         """Define function to save the generated token into database."""
         raise NotImplementedError()
 
-    def generate_token(self, grant_type, client, user=None, scope=None,
-                       expires_in=None, include_refresh_token=True):
+    def generate_token(
+        self,
+        grant_type,
+        client,
+        user=None,
+        scope=None,
+        expires_in=None,
+        include_refresh_token=True,
+    ):
         """Generate the token dict.
 
         :param grant_type: current requested grant_type.
@@ -49,40 +59,57 @@ class AuthorizationServer(object):
         func = self._token_generators.get(grant_type)
         if not func:
             # default generator for all grant types
-            func = self._token_generators.get('default')
+            func = self._token_generators.get("default")
         if not func:
-            raise RuntimeError('No configured token generator')
+            raise RuntimeError("No configured token generator")
 
         return func(
-            grant_type=grant_type, client=client, user=user, scope=scope,
-            expires_in=expires_in, include_refresh_token=include_refresh_token)
+            grant_type=grant_type,
+            client=client,
+            user=user,
+            scope=scope,
+            expires_in=expires_in,
+            include_refresh_token=include_refresh_token,
+        )
 
     def register_token_generator(self, grant_type, func):
         """Register a function as token generator for the given ``grant_type``.
         Developers MUST register a default token generator with a special
         ``grant_type=default``::
 
-            def generate_bearer_token(grant_type, client, user=None, scope=None,
-                                      expires_in=None, include_refresh_token=True):
-                token = {'token_type': 'Bearer', 'access_token': ...}
+            def generate_bearer_token(
+                grant_type,
+                client,
+                user=None,
+                scope=None,
+                expires_in=None,
+                include_refresh_token=True,
+            ):
+                token = {"token_type": "Bearer", "access_token": ...}
                 if include_refresh_token:
-                    token['refresh_token'] = ...
+                    token["refresh_token"] = ...
                 ...
                 return token
 
-            authorization_server.register_token_generator('default', generate_bearer_token)
+
+            authorization_server.register_token_generator(
+                "default", generate_bearer_token
+            )
 
         If you register a generator for a certain grant type, that generator will only works
         for the given grant type::
 
-            authorization_server.register_token_generator('client_credentials', generate_bearer_token)
+            authorization_server.register_token_generator(
+                "client_credentials",
+                generate_bearer_token,
+            )
 
         :param grant_type: string name of the grant type
         :param func: a function to generate token
         """
         self._token_generators[grant_type] = func
 
-    def authenticate_client(self, request, methods, endpoint='token'):
+    def authenticate_client(self, request, methods, endpoint="token"):
         """Authenticate client via HTTP request information with the given
         methods, such as ``client_secret_basic``, ``client_secret_post``.
         """
@@ -104,13 +131,15 @@ class AuthorizationServer(object):
         an example for this method::
 
             def authenticate_client_via_custom(query_client, request):
-                client_id = request.headers['X-Client-Id']
+                client_id = request.headers["X-Client-Id"]
                 client = query_client(client_id)
                 do_some_validation(client)
                 return client
 
+
             authorization_server.register_client_auth_method(
-                'custom', authenticate_client_via_custom)
+                "custom", authenticate_client_via_custom
+            )
         """
         if self._client_auth is None and self.query_client:
             self._client_auth = ClientAuthentication(self.query_client)
@@ -127,7 +156,7 @@ class AuthorizationServer(object):
         """
         raise NotImplementedError()
 
-    def create_oauth2_request(self, request):
+    def create_oauth2_request(self, request) -> OAuth2Request:
         """This method MUST be implemented in framework integrations. It is
         used to create an OAuth2Request instance.
 
@@ -136,7 +165,7 @@ class AuthorizationServer(object):
         """
         raise NotImplementedError()
 
-    def create_json_request(self, request):
+    def create_json_request(self, request) -> JsonRequest:
         """This method MUST be implemented in framework integrations. It is
         used to create an HttpRequest instance.
 
@@ -172,20 +201,26 @@ class AuthorizationServer(object):
         :param grant_cls: a grant class.
         :param extensions: extensions for the grant class.
         """
-        if hasattr(grant_cls, 'check_authorization_endpoint'):
+        if hasattr(grant_cls, "check_authorization_endpoint"):
             self._authorization_grants.append((grant_cls, extensions))
-        if hasattr(grant_cls, 'check_token_endpoint'):
+        if hasattr(grant_cls, "check_token_endpoint"):
             self._token_grants.append((grant_cls, extensions))
 
-    def register_endpoint(self, endpoint_cls):
+    def register_endpoint(self, endpoint):
         """Add extra endpoint to authorization server. e.g.
         RevocationEndpoint::
 
             authorization_server.register_endpoint(RevocationEndpoint)
 
-        :param endpoint_cls: A endpoint class
+        :param endpoint_cls: A endpoint class or instance.
         """
-        self._endpoints[endpoint_cls.ENDPOINT_NAME] = endpoint_cls(self)
+        if isinstance(endpoint, type):
+            endpoint = endpoint(self)
+        else:
+            endpoint.server = self
+
+        endpoints = self._endpoints.setdefault(endpoint.ENDPOINT_NAME, [])
+        endpoints.append(endpoint)
 
     def get_authorization_grant(self, request):
         """Find the authorization grant for current request.
@@ -193,7 +228,7 @@ class AuthorizationServer(object):
         :param request: OAuth2Request instance.
         :return: grant instance
         """
-        for (grant_cls, extensions) in self._authorization_grants:
+        for grant_cls, extensions in self._authorization_grants:
             if grant_cls.check_authorization_endpoint(request):
                 return _create_grant(grant_cls, extensions, request, self)
         raise UnsupportedResponseTypeError(request.response_type)
@@ -206,6 +241,7 @@ class AuthorizationServer(object):
         request.user = end_user
 
         grant = self.get_authorization_grant(request)
+        grant.validate_no_multiple_request_parameter(request)
         grant.validate_consent_request()
         return grant
 
@@ -215,7 +251,7 @@ class AuthorizationServer(object):
         :param request: OAuth2Request instance.
         :return: grant instance
         """
-        for (grant_cls, extensions) in self._token_grants:
+        for grant_cls, extensions in self._token_grants:
             if grant_cls.check_token_endpoint(request):
                 return _create_grant(grant_cls, extensions, request, self)
         raise UnsupportedGrantTypeError(request.grant_type)
@@ -230,12 +266,15 @@ class AuthorizationServer(object):
         if name not in self._endpoints:
             raise RuntimeError(f'There is no "{name}" endpoint.')
 
-        endpoint = self._endpoints[name]
-        request = endpoint.create_endpoint_request(request)
-        try:
-            return self.handle_response(*endpoint(request))
-        except OAuth2Error as error:
-            return self.handle_error_response(request, error)
+        endpoints = self._endpoints[name]
+        for endpoint in endpoints:
+            request = endpoint.create_endpoint_request(request)
+            try:
+                return self.handle_response(*endpoint(request))
+            except ContinueIteration:
+                continue
+            except OAuth2Error as error:
+                return self.handle_error_response(request, error)
 
     def create_authorization_response(self, request=None, grant_user=None):
         """Validate authorization request and create authorization response.
@@ -245,7 +284,9 @@ class AuthorizationServer(object):
             it is None.
         :returns: Response
         """
-        request = self.create_oauth2_request(request)
+        if not isinstance(request, OAuth2Request):
+            request = self.create_oauth2_request(request)
+
         try:
             grant = self.get_authorization_grant(request)
         except UnsupportedResponseTypeError as error:
@@ -254,9 +295,12 @@ class AuthorizationServer(object):
         try:
             redirect_uri = grant.validate_authorization_request()
             args = grant.create_authorization_response(redirect_uri, grant_user)
-            return self.handle_response(*args)
+            response = self.handle_response(*args)
         except OAuth2Error as error:
-            return self.handle_error_response(request, error)
+            response = self.handle_error_response(request, error)
+
+        grant.execute_hook("after_authorization_response", response)
+        return response
 
     def create_token_response(self, request=None):
         """Validate token request and create token response.
