@@ -1,22 +1,20 @@
 from authlib.consts import default_json_headers
 from authlib.jose import JoseError
-from ..rfc7591.claims import ClientMetadataClaims
-from ..rfc6749 import scope_to_list
+
 from ..rfc6749 import AccessDeniedError
 from ..rfc6749 import InvalidClientError
 from ..rfc6749 import InvalidRequestError
 from ..rfc6749 import UnauthorizedClientError
 from ..rfc7591 import InvalidClientMetadataError
+from ..rfc7591.claims import ClientMetadataClaims
 
 
 class ClientConfigurationEndpoint:
-    ENDPOINT_NAME = 'client_configuration'
+    ENDPOINT_NAME = "client_configuration"
 
-    #: The claims validation class
-    claims_class = ClientMetadataClaims
-
-    def __init__(self, server):
+    def __init__(self, server=None, claims_classes=None):
         self.server = server
+        self.claims_classes = claims_classes or [ClientMetadataClaims]
 
     def __call__(self, request):
         return self.create_configuration_response(request)
@@ -36,20 +34,25 @@ class ClientConfigurationEndpoint:
             # with HTTP 401 Unauthorized and the registration access token used to
             # make this request SHOULD be immediately revoked.
             self.revoke_access_token(request, token)
-            raise InvalidClientError(status_code=401)
+            raise InvalidClientError(
+                status_code=401, description="The client does not exist on this server."
+            )
 
         if not self.check_permission(client, request):
             # If the client does not have permission to read its record, the server
             # MUST return an HTTP 403 Forbidden.
-            raise UnauthorizedClientError(status_code=403)
+            raise UnauthorizedClientError(
+                status_code=403,
+                description="The client does not have permission to read its record.",
+            )
 
         request.client = client
 
-        if request.method == 'GET':
+        if request.method == "GET":
             return self.create_read_client_response(client, request)
-        elif request.method == 'DELETE':
+        elif request.method == "DELETE":
             return self.create_delete_client_response(client, request)
-        elif request.method == 'PUT':
+        elif request.method == "PUT":
             return self.create_update_client_response(client, request)
 
     def create_endpoint_request(self, request):
@@ -63,27 +66,27 @@ class ClientConfigurationEndpoint:
     def create_delete_client_response(self, client, request):
         self.delete_client(client, request)
         headers = [
-            ('Cache-Control', 'no-store'),
-            ('Pragma', 'no-cache'),
+            ("Cache-Control", "no-store"),
+            ("Pragma", "no-cache"),
         ]
-        return 204, '', headers
+        return 204, "", headers
 
     def create_update_client_response(self, client, request):
         # The updated client metadata fields request MUST NOT include the
         # 'registration_access_token', 'registration_client_uri',
         # 'client_secret_expires_at', or 'client_id_issued_at' fields
         must_not_include = (
-            'registration_access_token',
-            'registration_client_uri',
-            'client_secret_expires_at',
-            'client_id_issued_at',
+            "registration_access_token",
+            "registration_client_uri",
+            "client_secret_expires_at",
+            "client_id_issued_at",
         )
         for k in must_not_include:
             if k in request.data:
                 raise InvalidRequestError()
 
         # The client MUST include its 'client_id' field in the request
-        client_id = request.data.get('client_id')
+        client_id = request.data.get("client_id")
         if not client_id:
             raise InvalidRequestError()
         if client_id != client.get_client_id():
@@ -92,8 +95,8 @@ class ClientConfigurationEndpoint:
         # If the client includes the 'client_secret' field in the request,
         # the value of this field MUST match the currently issued client
         # secret for that client.
-        if 'client_secret' in request.data:
-            if not client.check_client_secret(request.data['client_secret']):
+        if "client_secret" in request.data:
+            if not client.check_client_secret(request.data["client_secret"]):
                 raise InvalidRequestError()
 
         client_metadata = self.extract_client_metadata(request)
@@ -102,56 +105,22 @@ class ClientConfigurationEndpoint:
 
     def extract_client_metadata(self, request):
         json_data = request.data.copy()
-        options = self.get_claims_options()
-        claims = self.claims_class(json_data, {}, options, self.get_server_metadata())
+        client_metadata = {}
+        server_metadata = self.get_server_metadata()
+        for claims_class in self.claims_classes:
+            options = (
+                claims_class.get_claims_options(server_metadata)
+                if server_metadata
+                else {}
+            )
+            claims = claims_class(json_data, {}, options, server_metadata)
+            try:
+                claims.validate()
+            except JoseError as error:
+                raise InvalidClientMetadataError(error.description) from error
 
-        try:
-            claims.validate()
-        except JoseError as error:
-            raise InvalidClientMetadataError(error.description)
-        return claims.get_registered_claims()
-
-    def get_claims_options(self):
-        metadata = self.get_server_metadata()
-        if not metadata:
-            return {}
-
-        scopes_supported = metadata.get('scopes_supported')
-        response_types_supported = metadata.get('response_types_supported')
-        grant_types_supported = metadata.get('grant_types_supported')
-        auth_methods_supported = metadata.get('token_endpoint_auth_methods_supported')
-        options = {}
-        if scopes_supported is not None:
-            scopes_supported = set(scopes_supported)
-
-            def _validate_scope(claims, value):
-                if not value:
-                    return True
-                scopes = set(scope_to_list(value))
-                return scopes_supported.issuperset(scopes)
-
-            options['scope'] = {'validate': _validate_scope}
-
-        if response_types_supported is not None:
-            response_types_supported = set(response_types_supported)
-
-            def _validate_response_types(claims, value):
-                return response_types_supported.issuperset(set(value))
-
-            options['response_types'] = {'validate': _validate_response_types}
-
-        if grant_types_supported is not None:
-            grant_types_supported = set(grant_types_supported)
-
-            def _validate_grant_types(claims, value):
-                return grant_types_supported.issuperset(set(value))
-
-            options['grant_types'] = {'validate': _validate_grant_types}
-
-        if auth_methods_supported is not None:
-            options['token_endpoint_auth_method'] = {'values': auth_methods_supported}
-
-        return options
+            client_metadata.update(**claims.get_registered_claims())
+        return client_metadata
 
     def introspect_client(self, client):
         return {**client.client_info, **client.client_metadata}
@@ -179,7 +148,7 @@ class ClientConfigurationEndpoint:
         Developers MUST implement this method in subclass::
 
             def authenticate_token(self, request):
-                auth = request.headers.get('Authorization')
+                auth = request.headers.get("Authorization")
                 return get_token_by_auth(auth)
 
         :return: token instance
@@ -191,7 +160,7 @@ class ClientConfigurationEndpoint:
         Developers MUST implement this method in subclass::
 
             def authenticate_client(self, request):
-                client_id = request.data.get('client_id')
+                client_id = request.data.get("client_id")
                 return Client.get(client_id=client_id)
 
         :return: client instance
@@ -237,7 +206,9 @@ class ClientConfigurationEndpoint:
         in subclass::
 
             def update_client(self, client, client_metadata, request):
-                client.set_client_metadata({**client.client_metadata, **client_metadata})
+                client.set_client_metadata(
+                    {**client.client_metadata, **client_metadata}
+                )
                 client.save()
                 return client
 
@@ -246,7 +217,6 @@ class ClientConfigurationEndpoint:
         :param request: formatted request instance
         :return: client instance
         """
-
         raise NotImplementedError()
 
     def get_server_metadata(self):
